# Ω*
#               ■          ■■■■■  
#               ■         ■■   ■■ 
#               ■        ■■     ■ 
#               ■        ■■       
#     ■■■■■     ■        ■■■      
#    ■■   ■■    ■         ■■■     
#   ■■     ■■   ■          ■■■■   
#   ■■     ■■   ■            ■■■■ 
#   ■■■■■■■■■   ■              ■■■
#   ■■          ■               ■■
#   ■■          ■               ■■
#   ■■     ■    ■        ■■     ■■
#    ■■   ■■    ■   ■■■  ■■■   ■■ 
#     ■■■■■     ■   ■■■    ■■■■■


"""
Moca System Core Module
モカシステムコアモジュール
茉客系统核心模块

[MocaCore]

Copyright (c) 2020.1.17 [el.ideal-ideas]

This software is released under the MIT License. see LICENSE.txt.

https://www.el-ideal-ideas.com
"""


# -- Imports --------------------------------------------------------------------------

from Crypto.Cipher import AES
from Crypto.Hash import SHA256
from Crypto import Random

# -------------------------------------------------------------------------- Imports --

# -- Private Functions --------------------------------------------------------------------------


def __create_aes(password: str,
                 iv: bytes):
    """
    create aes object
    :param password: password
    :param iv: initialization vector
    :return: aes object
    """
    sha256_hash = SHA256.new()
    sha256_hash.update(password.encode())
    hashed_key = sha256_hash.digest()
    return AES.new(hashed_key, AES.MODE_CFB, iv)

# -------------------------------------------------------------------------- Private Functions --

# -- Public Functions --------------------------------------------------------------------------


def encrypt(data: bytes,
            password: str) -> bytes:
    """
    encrypt data
    :param data: plain data
    :param password: password
    :return: encrypted data
    """
    iv = Random.new().read(AES.block_size)
    return iv + __create_aes(password, iv).encrypt(data)


def decrypt(data: bytes,
            password: str) -> bytes:
    """
    decrypt data
    :param data: encrypted data
    :param password: password
    :return: plain data
    """
    iv, cipher = data[:AES.block_size], data[AES.block_size:]
    return __create_aes(password, iv).decrypt(cipher)

# -------------------------------------------------------------------------- Public Functions --
